@@ -4,12 +4,14 @@ import signal
 import re
 import json
 
-from .checks import detectandcheck, allchecks
-from .checks import checkrsa, checkcrt, checksshpubkey
+from .checks import detectandcheck, defaultchecks, allchecks
+from .checks import checkrsa, checkcrt, checksshpubkey, checkpubkey
 from .allkeys import urllookup, loadextrabl
 from .scanssh import scanssh
 from .scantls import scantls
 from .update import update_bl
+from .dkim import parsedkim
+from . import __version__
 
 MAXINPUTSIZE = 10000
 
@@ -31,11 +33,11 @@ def _printresults(key, where, args):
     if "bits" in key:
         kn += f"[{key['bits']}]"
     if key["type"] == "unsupported":
-        print(f"Warning: Unsupported key type, {where}", file=sys.stderr)
+        print(f"WARNING: Unsupported key type, {where}", file=sys.stderr)
     elif key["type"] == "unparseable":
-        print(f"Warning: Unparseable input, {where}", file=sys.stderr)
+        print(f"WARNING: Unparseable input, {where}", file=sys.stderr)
     elif key["type"] == "notfound":
-        print(f"Warning: No key found, {where}", file=sys.stderr)
+        print(f"WARNING: No key found, {where}", file=sys.stderr)
     elif args.verbose or args.all:
         if key["results"] == {}:
             print(f"{kn} key ok, {where}")
@@ -48,6 +50,8 @@ def _printresults(key, where, args):
             url = urllookup(result["blid"], result["lookup"])
             if url:
                 print(url)
+            else:
+                sys.stderr.write("ERROR: URL lookup failed, not found\n")
         if args.verbose and "debug" in result:
             print(result["debug"])
         if args.verbose and "p" in result:
@@ -58,7 +62,10 @@ def _printresults(key, where, args):
 
 def runcli():
     global count
-    signal.signal(signal.SIGHUP, _sighandler)
+    try:
+        signal.signal(signal.SIGHUP, _sighandler)
+    except AttributeError:  # for OSes without SIGHUP (e.g. Windows)
+        pass
 
     ap = argparse.ArgumentParser()
     ap.add_argument(
@@ -80,9 +87,24 @@ def runcli():
     ap.add_argument(
         "--ssh-lines", action="store_true", help="Input file is list of ssh public keys"
     )
+    ap.add_argument("--dkim", action="store_true", help="Scan DKIM records (in files)")
+    ap.add_argument(
+        "--dkim-dns",
+        action="store_true",
+        help="Scan DKIM DNS record (hostnames instead of files)",
+    )
     ap.add_argument("-a", "--all", action="store_true", help="Show all keys")
+    ap.add_argument(
+        "-w",
+        "--warnings",
+        action="store_true",
+        help="Enable extra warnings (keysize etc.)",
+    )
     ap.add_argument("-v", "--verbose", action="store_true", help="Verbose output")
     ap.add_argument("-j", "--json", action="store_true", help="JSON output")
+    ap.add_argument(
+        "-q", "--quiet", action="store_true", help="Quiet output (for update commands)"
+    )
     ap.add_argument(
         "-u", "--url", action="store_true", help="Show private key URL if possible"
     )
@@ -113,6 +135,7 @@ def runcli():
         help="Scan SSH (pass hostnames or IPs instead of files)",
     )
     ap.add_argument("--ssh-ports", default="22", help="SSH ports (comma-separated)")
+    ap.add_argument("--version", action="version", version=__version__)
     args = ap.parse_args()
 
     if (
@@ -122,14 +145,16 @@ def runcli():
     ):
         sys.exit("Multiple input format parameters cannot be combined.")
 
-    if (args.moduli or args.crt_lines or args.ssh_lines) and (args.tls or args.ssh):
+    if (args.moduli or args.crt_lines or args.ssh_lines) and (
+        args.tls or args.ssh or args.dkim_dns
+    ):
         sys.exit("Scan modes and input file modes cannot be combined.")
 
     if args.update_bl_and_urls:
-        update_bl(lookup=True)
+        update_bl(lookup=True, quiet=args.quiet)
         sys.exit()
     if args.update_bl:
-        update_bl()
+        update_bl(quiet=args.quiet)
         sys.exit()
 
     if args.list:
@@ -152,7 +177,10 @@ def runcli():
             if c not in allchecks:
                 sys.exit(f"{c} is not a valid check")
     else:
-        userchecks = allchecks.keys()
+        if args.warnings:
+            userchecks = allchecks.keys()
+        else:
+            userchecks = defaultchecks.keys()
 
     if args.tls:
         ports = [int(p) for p in args.tls_ports.split(",")]
@@ -170,14 +198,33 @@ def runcli():
                 for k in keys:
                     _printresults(k, f"ssh:{host}:{port}", args)
 
-    if args.ssh or args.tls:
+    if args.dkim_dns:
+        try:
+            import dns.resolver
+        except ModuleNotFoundError:
+            sys.stderr.write("ERROR: DKIM DNS record scanning needs dnspython\n")
+            sys.exit(1)
+        for host in args.infiles:
+            try:
+                records = dns.resolver.resolve(host, "TXT").response
+            except (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer):
+                sys.stderr.write(f"No TXT record for {host} found\n")
+                continue
+            for record in records.answer[-1]:
+                dk = b"".join(record.strings).decode()
+                key = parsedkim(dk)
+                if key:
+                    r = checkpubkey(key, checks=userchecks)
+                    _printresults(r, host, args)
+
+    if args.ssh or args.tls or args.dkim_dns:
         sys.exit(1)
 
     for fn in args.infiles:
         if fn == "-":
             f = sys.stdin
         else:
-            f = open(fn)
+            f = open(fn, errors="replace")
         if args.moduli:
             for line in f:
                 count += 1
@@ -210,6 +257,16 @@ def runcli():
                 _printresults(r, desc, args)
                 lno += 1
                 count += 1
+        elif args.dkim:
+            lno = 0
+            for line in f:
+                desc = f"{fn}[{lno}]"
+                key = parsedkim(line)
+                if key:
+                    r = checkpubkey(key, checks=userchecks)
+                    _printresults(r, desc, args)
+                    count += 1
+                lno += 1
         else:
             fcontent = f.read(MAXINPUTSIZE)
             r = detectandcheck(fcontent, checks=userchecks)
